#include <iostream>
#include <string>
#include <cmath>
#include <time.h>

using namespace std;

class Stadt {
  private:
    string name;
    int entfernungen[6];
  public:
    void init( string name, int entfernungen[] );
    inline string getName();
    inline int getEntfernung( int index );
};

class TSPDeutschland {
  private:
    Stadt deutschland[6];
    Stadt* weg[6];
    void generiere( Stadt* neuerWeg[] );
    int kosten( Stadt* weg[] );
    bool pruefe( int kostenNeuerWeg, int kostenWeg, float temperatur );
  public:
    void werteInitialisieren();
    void hauptalgorithmus();
    void ausgabeErgebnis();
};

int main() {
  srand( (unsigned)time( NULL ) );
	
  TSPDeutschland SimAnn;

  SimAnn.werteInitialisieren();
  SimAnn.hauptalgorithmus();
  SimAnn.ausgabeErgebnis();

  return 0;
}

void Stadt::init( string name, int entfernungen[] ) {
  this->name = name;
  for( int i = 0; i < 6; i++ )
    this->entfernungen[i] = entfernungen[i];
}

inline string Stadt::getName() {
  return name;
}

inline int Stadt::getEntfernung( int index ) {
  return entfernungen[index];
}

void TSPDeutschland::generiere( Stadt* neuerWeg[] ) {
  Stadt* temp;
  int StadtA, StadtB;
	
  for( int i = 0; i < 6; i++ )
    neuerWeg[i] = weg[i];

  do {
    StadtA =  int( rand() / 32767.0 * 6.0 );
    StadtB =  int( rand() / 32767.0 * 6.0 );
  } while( StadtA > 5 || StadtB > 5 || StadtA == StadtB );

  temp = neuerWeg[StadtA];
  neuerWeg[StadtA] = neuerWeg[StadtB];
  neuerWeg[StadtB] = temp;
}

int TSPDeutschland::kosten( Stadt* weg[] ) {
  int gesamt = 0;
	
  for( int i = 0; i < 5; i++ )
    gesamt += weg[i]->getEntfernung( weg[i+1] - deutschland );

  gesamt += weg[5]->getEntfernung( weg[0] - deutschland );

  return gesamt;
}

bool TSPDeutschland::pruefe( int kostenNeuerWeg, int kostenWeg, float temperatur )  {
  int diff_cost = kostenNeuerWeg - kostenWeg;
  double y = exp( -diff_cost / temperatur );
  double r = rand() / 32767.0;

  if( r < y )
    return true;
  else
    return false; 
}


void TSPDeutschland::werteInitialisieren() {
  int entfernungen[6];

  entfernungen[0] = 0;
  entfernungen[1] = 91;
  entfernungen[2] = 80;
  entfernungen[3] = 259;
  entfernungen[4] = 70;
  entfernungen[5] = 121;
  deutschland[0].init( "Aachen", entfernungen );

  entfernungen[0] = 91;
  entfernungen[1] = 0;
  entfernungen[2] = 77;
  entfernungen[3] = 175;
  entfernungen[4] = 27;
  entfernungen[5] = 84;
  deutschland[1].init( "Bonn", entfernungen );

  entfernungen[0] = 80;
  entfernungen[1] = 77;
  entfernungen[2] = 0;
  entfernungen[3] = 232;
  entfernungen[4] = 47;
  entfernungen[5] = 29;
  deutschland[2].init( "Duesseldorf", entfernungen );

  entfernungen[0] = 259;
  entfernungen[1] = 175;
  entfernungen[2] = 232;
  entfernungen[3] = 0;
  entfernungen[4] = 189;
  entfernungen[5] = 236;
  deutschland[3].init( "Frankfurt", entfernungen );

  entfernungen[0] = 70;
  entfernungen[1] = 27;
  entfernungen[2] = 47;
  entfernungen[3] = 189;
  entfernungen[4] = 0;
  entfernungen[5] = 55;
  deutschland[4].init( "Koeln", entfernungen );
	
  entfernungen[0] = 121;
  entfernungen[1] = 84;
  entfernungen[2] = 29;
  entfernungen[3] = 236;
  entfernungen[4] = 55;
  entfernungen[5] = 0;
  deutschland[5].init( "Wuppertal", entfernungen );

  for( int i = 0; i < 6; i++ )
    weg[i] = &deutschland[i];	
}

void TSPDeutschland::hauptalgorithmus() {
  float temperatur = 10;
  int schritte;
  Stadt* neuerWeg[6];

  while (temperatur > 0.1) {
    schritte = 0;

    while( schritte < 10 ) {
      generiere( neuerWeg ); 	
      if( pruefe( kosten( neuerWeg ), kosten( weg ), temperatur ) ) {	
        for( int i = 0; i < 6; i++ )
          weg[i] = neuerWeg[i]; 
      }
      schritte++;
    }
    temperatur = temperatur * 0.9; 
  }
}

void TSPDeutschland::ausgabeErgebnis() {
  cout << "Gefundenes Ergebnis: " << endl;
  for( int i = 0; i < 6; i++ )
    cout << weg[i]->getName() << "->";

  cout << endl << endl << "Gesamtstrecke: " << kosten(weg) << endl;
}
